# Erstellen einer GitLab-Page

Sie finden auf [dieser Seite](https://gitlab.opencode.de/open-code/hilfe/opencode-mkdocs) unser Beispiel-Projekt zum Erstellen einer eigenen statischen Webseite auf openCode.

In dem Projekt befindet sich ein Beispiel einer GitLab-Page mit einem angepassten openCode-Theme.

Das Theme entspricht dabei folgendem Screenshot:
![Screenshot der Startseite im Browser](assets/images/gitlab-pages/2025-02-19_12-40-54_Screenshot.png)

## Schritte zum Erstellen einer eigenen Seite

### 1. Einstellung des Seiten-Generators

In der Datei `mkdocs.yml` kann der Seiten-Generator für die statische Webseite eingerichtet werden.

- [ ] Festlegen von dem Namen der Seite:

  ```yaml
  site_name: openCode-Pages Beispiel    # Hier kann der Name der Seite geändert werden
  ```

- [ ] Verlinkung des Projekts auf dem openCode GitLab:

  ```yaml
  repo_url: https://gitlab.opencode.de/open-code/hilfe/opencode-mkdocs    # URL zum GitLab-Projekt (bitte anpassen)
  repo_name: openCode-Pages Beispiel    # Name des GitLab-Projekts (bitte anpassen)
  ```

- [ ] Verlinkung eines Logos und eines Icons:
  Im Standard wird die Datei `assets/logo.svg` und `assets/icon.svg` verlinkt. Daher am Besten die daher die Dateien durch das eigene Logo bzw. Icon zu ersetzen.

  ```yaml
  theme:
    logo: assets/logo.svg   # Pfad zum Logo der Webseite (bitte anpassen)
    favicon: assets/icon.svg    # Pfad zu dem Icon der Webseite (bitte anpassen)
  ```

### 2. Rechtliche Hinweise anpassen

- [ ] **Hinzufügen eines Impressums:**
  Die Datei `docs/legal/impressum.md` beinhaltet das Impressum der Seite. Diese Datei nun am besten an den eigenen Bedarf anpassen.
- [ ] **Hinzufügen von Hinweisen zum Datenschutz:**
  Die Datei[`docs/legal/datenschutz.md` beinhaltet die Hinweise zum Datenschutz der Seite. Diese Datei nun am besten an den eigenen Bedarf anpassen.
- [ ] **Copyright-Notiz anpassen:**
  In der Datei `mkdocs.yml` kann die Copyright-Notiz angepasst werden:

  ```yaml
  copyright: © 2025 openCode    # Copyright-Notiz (bitte anpassen)
  ```

### 3. Startseite anpassen

- [ ] Die Startseite `docs/index.md` an den eigenen Bedarf anpassen.

### 4. Eigene Inhalte hinzufügen

- [ ] Anlegen von neuen Seiten über das Erstellen neuer Markdow-Dateien (`.md`) im Ordner `docs/`
- [ ] Damit diese neuen Seiten auch in die Navigation aufgenommen werden, müssen diese anschließend in der Datei `mkdocs.yml` wie folgt angegeben werden:
    ```yaml
    nav:
        - index.md
        - about.md    # Hinzufügen einer neuen "Über uns"-Seite
    ```

### Ergänzende Hinweise

- **Style-Anpassungen** über die Datei `docs/style/extra.css`
- Es befindet sich ein **Glossar** wichtiger Begriffe in der Datei `includes/abbreviations.md`. Diese werden auf allen Seiten hervorgehoben und in einem Tooltip (welches auf den Glossar zugreift) erläutert.
- Die **GitLab CI/CD-Pipeline** kann in der Datei `.gitlab-ci.yml` angepasst werden
- **Bilder** sollten am Besten im Ordner `docs/assets/images/` gespeichert werden.
