# Aufnahme in das Softwareverzeichnis

## Erstellung einer publiccode.yml-Datei

Auf dieser Seite finden Sie eine Anleitung zur Erstellung einer publiccode.yml-Datei.

Eine `publiccode.yml`-Datei ist notwendig, damit Projekte in unser Software-Verzeichnis aufgenommen werden.
Die `publiccode.yml`-Datei muss sich dabei im obersten Ordner ihres Projekts auf gitlab.opencode.de befinden.
Es werden alle Repositories auf neue `publiccode.yml`-Dateien geprüft und danach in das [openCode-Softwareverzeichnis](https://opencode.de/de/software) aufgenommen.

![Infografik zur Veröffentlichung eines Projekts im openCode-Softwareverzeichnis](assets/images/softwareverzeichnis/infografik.png)

Sie können zum Erstellen einer `publiccode.yml`-Datei ...

- entweder unseren **Generator nutzen**, welcher auf Nutzer:innenfreundlicher Art und Weise hilft eine standard-konforme Datei zu generieren
    - [Zum publiccode.yml-Editor](https://editor.opencode.de/)

        !!! info
            Der Editor kann sie auf Fehler in Ihrer `publiccode.yml` Datei hinweisen. Drücken sie hierzu einfach, nachdem Sie ihre `publiccode.yml`-Datei hochgeladen haben, auf die "Validate"-Schaltfläche.

- oder Sie **orientieren sich an folgender kommentierter publiccode.yml-Datei**:
    - [Download der kommentierten `publiccode.yml`](assets/code/publiccode.yml)

```yaml title="publiccode.yml"
--8<-- "publiccode.yml"
```

1. Version des publiccode.yml-Standards. Nur ändern, wenn sie wissen, was sie tun!
2. Dieser Schlüssel ist der "genericName", der sich auf die spezifische Kategorie bezieht, zu der die Software gehört. Sie können den generischen Namen normalerweise in einer Präsentation der Software finden, wenn Sie etwas wie folgt schreiben: "Software xxx ist ein yyy" Nennenswerte Beispiele sind zum Beispiel: "Texteditor", "Textverarbeitung", "Webbrowser", "Chat" und so weiter... Der generische Name darf bis zu 35 Zeichen lang sein.
3. Die URL ist meist ihre Projektseite auf `gitlab.opencode.de`
4. Der Pfad ihres Logo ist relativ von dem obersten Ordner ihres Projekts. Erlaubte Dateiformate sind: `.svg`, `.svgz` und `.png`
5. Mögliche Einträge: `web`, `windows`, `mac`, `linux`, `ios`, `android`
6. Beliebige Anzahl an Einträgen möglich.
7. Eine Liste der möglichen Kategorien finden sie auf folgender Webseite: https://yml.publiccode.tools/categories-list.html
8. Einer der folgenden Angaben: `concept`, `development`, `beta`, `stable`, `obsolete`
9. Eine der folgenden Angaben: `standalone/backend`, `standalone/desktop`, `standalone/iot`, `standalone/mobile`, `standalone/web`, `standalone/other`, `addon`, `library`, `configurationFiles`
19. Sprachen im Form eines IETF BCP 47 language tag (siehe https://en.wikipedia.org/wiki/IETF_language_tag)
11. Maximal 150 Zeichen (in Deutsch)
12. Dieser Schlüssel enthält eine längere Beschreibung der Software, zwischen 500 und 10000 Zeichen (auf Deutsch). Sie soll einem potenziellen Benutzer einen Überblick über die Fähigkeiten der Software für einen potenziellen Benutzer geben. Die Zielgruppe für diesen Text sollten die Benutzer der Software sein, nicht die Entwickler. Sie können sich diesen Text vorstellen Text als die Beschreibung der Software, die auf ihrer Website zu finden wäre (wenn sie eine hätte). Diese Beschreibung kann einige grundlegende Markdown-Elemente enthalten: ```*Kursiv*```, ```**fett**```, ```- Aufzählungen``` und ```[Links](#)```.
13. Pfad zu ihren Screenshots. Die Pfade sind ausgehend von dem obersten Ordner ihres Projekts
14. Maximal 150 Zeichen (in Englisch)
15. Dieser Schlüssel enthält eine längere Beschreibung der Software, zwischen 500 und 10000 Zeichen (auf Englisch). Sie soll einem potenziellen Benutzer einen Überblick über die Fähigkeiten der Software für einen potenziellen Benutzer geben. Die Zielgruppe für diesen Text sollten die Benutzer der Software sein, nicht die Entwickler. Sie können sich diesen Text vorstellen Text als die Beschreibung der Software, die auf ihrer Website zu finden wäre (wenn sie eine hätte). Diese Beschreibung kann einige grundlegende Markdown-Elemente enthalten: ```*Kursiv*```, ```**fett**```, ```- Aufzählungen``` und ```[Links](#)```.
16. Eine Liste der zugelassenen Lizenzen finden sie auf folgender Webseite: https://opencode.de/wissen/rechtssichere-nutzung/open-source-lizenzen#2.-Open-Source-Lizenzliste (gültig sind Einträge im Format der auf der Seite angegebenen SPDX-Identifier).
17. Eine der folgenden Angaben: `internal`, `contract`, `community`, `none`

Die Spezifikationen für den publiccode Standard können Sie folgender Webseite entnehmen: [https://yml.publiccode.tools/schema.core.html](https://yml.publiccode.tools/schema.core.html)

### Empfohlene Order-Struktur ihres Projekts

Wir empfehlen eine Ordner-Struktur, die ähnlich wie folgt aussieht:

``` { .txt .no-copy }
Mein-Projekt/
├── logo.svg
└── .opencode/
    └── screenshots/
        ├── sshot1.png
        ├── sshot2.png
        └── sshot3.png
```

## Häufige Probleme

### Das Logo oder die Screenshots meines Projekts werden nicht angezeigt?

Falls das Logo oder sind die Screenshots ihres Projekts nicht angezeigt werden, kann das daran an folgenden Fehlern liegen:

- **In der publiccode.yml-Datei ist ihr Logo oder ihre Screenshots noch nicht aufgeführt:**
    - **Hinzufügen eines Logos:** Fügen sie einen ähnlichen Eintrag, wie den folgenden, zu ihrer publiccode.yml-Datei hinzu (`logo.svg` entspricht dabei dem Dateinamen ihres Logos):

        ``` { .yaml .no-copy }
        logo: logo.svg
        ```

        !!! warning "Warnung"
            Die Logo-Datei muss sich in dem obersten Ordner ihres Projekts befinden.

    - **Hinzufügen von Screenshots:** Fügen sie im Bereich `description` unter dem Bereich der jeweiligen unterstützen Sprache (z.B. `de`) einen ähnlichen Eintrag, wie den folgenden, hinzu:

        ``` { .yaml .no-copy }
        description:
            de:
              screenshots:
                - .opencode/screenshots/sshot1.jpg
                - .opencode/screenshots/sshot2.jpg
                - .opencode/screenshots/sshot3.jpg
        ```

- **Sie nutzen ein von uns nicht unterstütztes Dateiformat:**
    - Gültige Dateiformate für Logos sind: `.svg`, `.svgz` und `.png`.
    - Gültige Dateiformate für Screenshots sind: `.png` und `.jpg.`
- **Der von ihnen angegebene Pfad zu ihrer Logo-Datei ist nicht korrekt:**
    - Überprüfen sie, ob der Ordner (ausgehend vom obersten Ordner ihres Projekts) existiert.
    - Schauen sie, ob sie die von uns empfohlene Ordner-Struktur verwenden: [Empfohlene Ordner-Struktur](#empfohlene-order-struktur-ihres-projekts)

### Ihr Projekt wird nicht in dem openCode Softwareverzeichnis angezeigt?

Dies liegt vermutlich daran, dass Sie entweder Fehler in Ihrer publiccode.yml-Datei haben, oder bestimme Einträge in Ihrer `publiccode.yml`-Datei vergessen haben.

Am Einfachsten überprüfen Sie Ihre `publiccode.yml`-Datei damit, wenn Sie diese in unseren [`publiccode.yml`-Editor](https://editor.opencode.de/) laden und danach auf die "Validate"-Schaltfläche klicken.

## publiccode.yml-Dateien aus der Community

Folgende Beispiele aus der Community können für Sie nützlich sein, um die Erstellung einer `publiccode.yml`-Datei nachvollziehen zu können:

- [`publiccode.yml` von openDesk](https://gitlab.opencode.de/bmi/opendesk/info/-/blob/main/publiccode.yml)
- [`publiccode.yml` der Smart Village App](https://gitlab.opencode.de/bad-belzig/smart-village-app/-/blob/1deefa789cfa7889062c9944513263cc124725fc/publiccode.yml)
- [`publiccode.yml` des Content-Management-Systems Government Site Builder](https://gitlab.opencode.de/bmi/government-site-builder-11/gsb11/-/blob/main/publiccode.yml)
- [`publiccode.yml` des Grüne-Welle-Assistent Leezenflow der Stadt Münster](https://gitlab.opencode.de/smart-city-muenster/leezenflow/leezenflow-code/-/blob/main/publiccode.yml)
- [`publiccode.yml` der Portalsoftware für regionales Kulturmarketing OpenCulturas](https://gitlab.opencode.de/openculturas/openculturas-distribution/-/blob/2.3.x/publiccode.yml)
